@@ -299,7 +299,7 @@ sub run {
             my $second_timestamp = $entries[1]->Timestamp;
 
             if ($first_timestamp && $second_timestamp) {
-                tag 'latest-debian-changelog-entry-without-new-date'
+                tag 'latest-changelog-entry-without-new-date'
                   unless (($first_timestamp - $second_timestamp) > 0
                     or lc($entries[0]->Distribution) eq 'unreleased');
             }
@@ -468,7 +468,7 @@ sub run {
 
         # Strip out all lines that contain the word spelling to avoid false
         # positives on changelog entries for spelling fixes.
-        $changes =~ s/^.*spelling.*\n//gm;
+        $changes =~ s/^.*(?:spelling|typo).*\n//gm;
         check_spelling($changes, $group->info->spelling_exceptions,
             $SPELLING_ERROR_CHANGELOG);
     }
